<?php

/**
 *    LIMACUTE - Linpro Mailserver Configuration Utility Eh...
 *
 *    Copyright (C) 2000-2001 Linpro AS
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    Contact: limacute@linpro.no or 
 *    Linpro AS, Waldemar Thranes gate 98 B, 0175 OSLO, Norway
 *    <ULR: http://www.linpro.no/ >
 *
 *
 * Description: A small collection of utility functions, most of them borrowed
 * from Squirrelmail.
 * Compilator: Knut Haugen <knuthaug@linpro.no>
 * version: $Id: functions.php,v 1.3 2001/12/06 16:44:33 andread Exp $
 *
 */

/**
 * Function: makeSalt
 *
 * Description: Make a unique (or pretty much unique) salt to use 
 * with crypt man encrypting passwords
 * Returns: A two char salt for standard des crypt()
 * Author: Knut Haugen 
 */
function makeSalt (){
    //randomize the mt-engine
    mt_randomize();
    
    //generate random string of 10 chars
    $string = "";
    $len = 10;
    
    //generate only [a-zA-Z0-9]
    for ($i = 0; $i < $len; $i++){
        if ($i % 2 == 0){
            $string .= chr(mt_rand(65,90));            
        }
        elseif ($i % 2 == 1){        
            $string .= chr(mt_rand(97,122));
        }
        else{
            $string .= chr(mt_rand(48,57));
        }
    }

    $string = base64_encode ($string);

    //pick two random chars from the string
    $salt = substr($string, mt_rand (0,$len), 1); 
    $salt .= substr($string, mt_rand (0,$len), 1); 
    return $salt;
}


/**
 * Function: makeRandomString
 *
 * Argument: length of string to create (default 100) 
 * Returns: base64_encoded randomstring
 * Note: Borrowed from Squirrelmail
 */
function makeRandomString ($length=100) {
    mt_randomize();
    
    $pad = '';
    for ($i = 0; $i < $length; $i++) {
        $pad .= chr(mt_rand(0,255));
    }
    
    return base64_encode($pad);
}

/**
 * Function: OneTimePadEncrypt
 *
 * Arguments: the password entered and the onetimepad
 * returns: base64_encoded XOR of the two arguments
 * Borrowed from Squirrelmail
 */
function OneTimePadEncrypt ($string, $epad) {
    $pad = base64_decode($epad);
    $encrypted = '';
    for ($i = 0; $i < strlen ($string); $i++) {
        $encrypted .= chr (ord($string[$i]) ^ ord($pad[$i]));
    }
    
    return base64_encode($encrypted);
}

/**
 * Function: OneTimePadDecrypt
 *
 * Arguments: the stored key and the onetimepad
 * returns: XOR of the two arguments, decrypted password
 * Borrowed from Squirrelmail
 */
function OneTimePadDecrypt ($string, $epad){
    $pad = base64_decode($epad);
    $encrypted = base64_decode ($string);
    $decrypted = '';
    for ($i = 0; $i < strlen ($encrypted); $i++) {
        $decrypted .= chr (ord($encrypted[$i]) ^ ord($pad[$i]));
    }
    
    return $decrypted;
}


/**
 * Function: mt_seed
 *
 * Description: randomixe the mt_rand() function
 * description: Seed the random number generator
 * Arguments: value to use as seed
 * Note: Borrowed from Squirrelmail
 */
function mt_seed($Val){
    // if mt_getrandmax() does not return a 2^n - 1 number,
    // this might not work well.  This uses $Max as a bitmask.
    $Max = mt_getrandmax();
    
    if (! is_int($Val)){
        if (function_exists('crc32')){
            $Val = crc32($Val);
        }
        else{
            $Str = $Val;
            $Pos = 0;
            $Val = 0;
            $Mask = $Max / 2;
            $HighBit = $Max ^ $Mask;
            while ($Pos < strlen($Str)){
                if ($Val & $HighBit){
                    $Val = (($Val & $Mask) << 1) + 1;
                }
                else{
                    $Val = ($Val & $Mask) << 1;
                }
                $Val ^= $Str[$Pos];
                $Pos ++;
            }
        }
    }
    
    if ($Val < 0)
        $Val *= -1;
    if ($Val = 0)
        return;
    
    mt_srand(($Val ^ mt_rand(0, $Max)) & $Max);
}

/**
 * Function: mt_randomize
 *
 * Description: initialize the random number generator, once
 * Arguments: none
 * returns: nothing
 * Borrowed from Squirrelmail
 */
function mt_randomize(){
    global $REMOTE_PORT, $REMOTE_ADDR, $UNIQUE_ID;
    static $randomized;
    
    if ($randomized)
        return;
    
    // Global   
    mt_seed((int)((double) microtime() * 1000000));
    mt_seed(md5($REMOTE_PORT . $REMOTE_ADDR . getmypid()));
      
      // getrusage
    if (function_exists('getrusage')) {
        // Avoid warnings with Win32
        $dat = @getrusage();
        if (isset($dat) && is_array($dat)){
            $Str = '';
            foreach ($dat as $k => $v){
                $Str .= $k . $v;
            }
            mt_seed(md5($Str));
        }
    }
      
    // Apache-specific
    mt_seed(md5($UNIQUE_ID));
    
    $randomized = 1;
}

function arraysort ($a, $b) {  
    return strcmp($a["domain"][0], $b["domain"][0]);
}

function usersort ($a, $b) {  
    return strcmp($a["uid"][0], $b["uid"][0]);
}

?>
